package tdd.tp03;

import java.io.IOException;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.input.TextInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;

class InterMapper
extends Mapper<Object, Text, IntWritable, IntWritable> {

	@Override
	public void map(Object o, Text in, Context ctx) 
			throws IOException, InterruptedException
	{
	
		IntWritable left = new IntWritable(0);
		IntWritable right = new IntWritable(1);
		
		String s = in.toString();
		String[] ints = s.split("[ ]+");
		if (ints.length == 2) {
		IntWritable i1 = new IntWritable(Integer.parseInt(ints[0]));
		IntWritable i2 = new IntWritable(Integer.parseInt(ints[1]));
		
		
		ctx.write(i1, left);
		ctx.write(i2, right);
		}
				
	};
	
}

class InterReducer
extends Reducer<IntWritable, IntWritable, IntWritable, NullWritable> {

	@Override
	public void reduce(IntWritable i, Iterable<IntWritable> sides,
						Context ctx) 
	throws IOException, InterruptedException {
		
		boolean left = false;
		boolean right = false;
		for(IntWritable side : sides) {
			
			int s = side.get();
			if (s == 0) left = true;
			if (s == 1) right = true;
			if (left && right) {
				ctx.write(i, NullWritable.get());
				break;
			};
		}
			
	}
	
}
	
public class DriverInter {

	
	
	public static void main(String[] args)
	
	{
		if (args.length < 2) System.exit(2);
		
		try {
			
			//Création d'une nouvelle configuration
			Configuration conf = new Configuration();

			Job job = Job.getInstance(conf, "tdd.tp03");
		    
					
			
			//Le nom de la classe 'main'
			job.setJarByClass(DriverInter.class);

			//Le format d'entrée du Map
			//TextInputFormat est un alias pour
			//InputFormat<LongWritable, Text>;
			
			job.setInputFormatClass(TextInputFormat.class);
			
			//Initialisation du Map
			job.setMapperClass(InterMapper.class);
			
			//Initialisation du Reduce
			job.setReducerClass(InterReducer.class);

			//Sortie du Map
			job.setMapOutputKeyClass(IntWritable.class);
			job.setMapOutputValueClass(IntWritable.class);
			
			//Sortie (du Reduce)
			job.setOutputKeyClass(IntWritable.class);
			job.setOutputValueClass(NullWritable.class);
						
			FileOutputFormat.setOutputPath(job, new Path(args[0]));
			
			for(int i = 1; i < args.length; i++)
				FileInputFormat.addInputPath(job, new Path(args[i]));
	
						
			
			System.exit(job.waitForCompletion(true) ?  0 : 1);
			
		} catch (Exception e) {
			System.err.println("Erreur : " + e);
		}
		
	}
	
}
